{% load i18n %}
<div class="submit-row">
	<input class="pfp-button-left" type="submit" value="{% trans 'Filter' %}" name="_filter" />
{% if show_save %}<input type="submit" value="{% trans 'Save' %}" class="default" name="_save" />{% endif %}
{% if show_save_and_continue %}<input type="submit" value="{% trans 'Save and continue editing' %}" name="_continue" />{% endif %}
</div>
