# Generated by Django 2.1.1 on 2018-09-21 06:11

import django.contrib.auth.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('perfieldperms', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='perfieldpermission',
            managers=[
                ('objects', django.contrib.auth.models.PermissionManager()),
            ],
        ),
    ]
